module Stacko
  class Application
    def self.builder
      @builder ||= Builder.new
    end

    def self.map(method_name, &block)
      builder.map(method_name, &block)
      class_eval <<-EOT, __FILE__, __LINE__ + 1
        def #{method_name}(*args)                  # def get_user(*args)
          execute(:#{method_name}, *args)          #   execute(:get_user, *args)
        end                                        # end
      EOT
    end

    def self.run(app)
      builder.run(app)
    end

    def self.use(middleware, *args, &block)
      builder.use(middleware, *args, &block)
    end

    def call(*args)
      execute(:call, *args)
    end

    private

    def execute(method_name, *args)
      env = {}
      env['METHOD_NAME'] = method_name
      env['ARGUMENTS'] = *args
  
      self.class.builder.call(env)
    end
  end
end