module Stacko
  class Builder
    def self.app(default_app = nil, &block)
      self.new(default_app, &block).to_app
    end
    
    def initialize(default_app = nil, &block)
      @run = default_app
      @use = []
      @map = nil
      instance_eval(&block) if block_given?
    end

    def use(middleware, *args, &block)
      if @map
        mapping = @map
        @map = nil
        @use << proc { |app| generate_map(app, mapping) }
      end
      @use << proc { |app| middleware.new(app, *args, &block) }
    end

    def run(app)
      @run = app
    end

    def map(method_name, &block)
      @map ||= {}
      @map[method_name] = block
    end

    def to_app
      app = @map ? generate_map(@run, @map) : @run
      fail "missing run or map statement" unless app
      app = @use.reverse.inject(app) { |a,e| e[a] }
      app
    end

    def call(env)
      to_app.call(env)
    end

    private

    def generate_map(default_app, mapping)
      mapped = default_app ? {:call => default_app} : {}
      mapping.each { |r,b| mapped[r] = self.class.new(default_app, &b) }
      MethodMap.new(mapped)
    end
  end
end